#!/usr/bin/env python3

import argparse

from junitparser import JUnitXml, TestSuite


def case_dictionary(report):
    return {
        (case.classname, case.name): case
        for suite in report
        for case in suite
    }


def compare_reports(path_before, path_after, path_out):
    before = JUnitXml.fromfile(path_before)
    after = JUnitXml.fromfile(path_after)
    before_cases = case_dictionary(before)
    after_cases = case_dictionary(after)

    changed_cases = TestSuite('tests_with_changed_result')

    for key, after_case in after_cases.items():
        if key in before_cases:
            before_case = before_cases[key]
            before_results = [type(r) for r in before_case.result]
            after_results = [type(r) for r in after_case.result]
            if set(before_results) != set(after_results):
                changed_cases.add_testcase(after_case)

    xml = JUnitXml()
    xml.add_testsuite(changed_cases)
    xml.update_statistics()
    xml.write(path_out, to_console=False)


def main():
    parser = argparse.ArgumentParser(
        description='Create a diff of two JUnit test reports. The resulting test report contains only '
                    'tests that are present in both of the input reports. In addition, their status has '
                    'to have changed in order for the tests to be included. '
                    'The status of the last input report is shown in the diff.'
    )
    parser.add_argument("before", help="Path of the initial XML report to compare.")
    parser.add_argument("after", help="Path of the re-run XML report to compare.")
    parser.add_argument("output", help='Path to write diff report to.')
    args = parser.parse_args()
    compare_reports(args.before, args.after, args.output)


if __name__ == '__main__':
    main()
